var app = {

    _nid: 0,

    db: null,
    ble: null,
    smbList: null,

    beacon_colors:  [ 'purple', 'black', 'white', 'cyan', 'blue', 'green', 'red' ],
    beacon_icons: [
        'fa-caret-up',
        'fa-key',
        'fa-credit-card',
        'fa-wallet'
    ],

    batterystate: [
        '<i class="fas fa-battery-empty"></i>',
        '<i class="fas fa-battery-quarter"></i>',
        '<i class="fas fa-battery-half"></i>',
        '<i class="fas fa-battery-three-quarters"></i>',
        '<i class="fas fa-battery-full"></i>',
        '<i class="fas fa-battery-full"></i>'
    ],

    connectionstate: {
        authorized: 'kapcsolódva',
        disconnected: 'nincs kapcsolat',
        handshake: 'azonosítás...',
        connection: 'kapcsolódás...'
    },

    // Application Constructor
    initialize: function() {

        app.db = window.localStorage;

        document.addEventListener('deviceready', this.onDeviceReady.bind(this), false);

        document.addEventListener('init', this.onOnsenInit, false);
        document.addEventListener('show', this.onOnsenShow, false);
        document.addEventListener('hide', this.onOnsenHide, false);

        window.fn = {};

        window.fn.open = function() {
            var menu = document.getElementById('mainmenu');
            var currentbag = app.currentSMB();

            if (currentbag) {
                $('#current-bag-title').html('SMB-'+currentbag.sn);
            } else {
                $('#current-bag-title').html('Nincs kiválasztott táska');
            }

            var content = document.getElementById('myNav');

            $('#mainmenu ons-list > ons-list-item').removeClass('disabled');
            $('#mainmenu ons-list > ons-list-item').removeClass('active');

            if (!currentbag) {
                /* disable menu items related to bag services */
                $('#mainmenu ons-list > ons-list-item[data-pageid="page-bag-beacon"]').addClass('disabled');
                // $('#mainmenu ons-list > ons-list-item[data-pageid="page-bag-map"]').addClass('disabled');
                $('#mainmenu ons-list > ons-list-item[data-pageid="page-bag-settings"]').addClass('disabled');
            }

            $('#mainmenu ons-list > ons-list-item').removeClass('active');
            $('#mainmenu ons-list > ons-list-item[data-pageid="'+content.topPage.id+'"]').addClass('active');

            menu.open();
        };

        window.fn.load = function(page, data, force = false) {
            var content = document.getElementById('myNav');
            var menu = document.getElementById('mainmenu');

            if (!force && page.substr(0,page.length-5) == content.topPage.id) {
                app.log('Already on the selected page');
                menu.close.bind(menu);
                return;
            }

            content.bringPageTop(page, data)
                .then( menu.close.bind(menu) )
                .catch( (err) => {
                    console.log(err);
                });
        };

        window.fn.pop = function() {
            var content = document.getElementById('myNav');

            if (!content) {
                cordova.plugins.backgroundMode.moveToBackground();
                return
            }

            // hide dialog if visible
            let _od = $('ons-dialog:visible');
            if (_od.length>0) {
                $('ons-dialog p:last-child ons-button:last-child').click();
                return
            }

            // pop a page when ther is at least one page in the stack
            if (content.pages.length>1) {
                content.popPage();
                return
            }

            // move app to background in case of no pages in the stack
            cordova.plugins.backgroundMode.moveToBackground();

        };

        ons.ready( () => {
            ons.disableDeviceBackButtonHandler();
            document.addEventListener('backbutton', function () {
                window.fn.pop();
            }, true);
        });
    },

    log: function() {
        console.log(...arguments);
    },

    /* EVENTS */

    // deviceready Event Handler
    onDeviceReady: function() {
        app.log('Device is ready');

        cordova.plugins.backgroundMode.enable();
        cordova.plugins.backgroundMode.overrideBackButton();
        cordova.plugins.backgroundMode.setDefaults({
            title: 'SMB',
            text: 'Watching after your SMB',
            icon: 'icon', // this will look for icon.png in platforms/android/res/drawable|mipmap
            color: 'FFFFFF',
            //resume: true,
            hidden: false,
            bigText: false
        })

        cordova.plugins.notification.local.setDefaults({
            smallicon: 'res://icon',
            icon: 'res://icon',
            led: { color: '#FF00FF', on: 500, off: 500 },
            vibrate: true
        });

        //cordova.plugins.backgroundMode.disableBatteryOptimizations();

        cordova.plugins.backgroundMode.on('activate', function() {
            cordova.plugins.backgroundMode.disableWebViewOptimizations();
        });

        //cordova.plugins.foregroundService.start('Smartbag', 'Watching after your SMB', 'icon', 1);

        window.plugins.insomnia.keepAwake();

        // Bluetooth driver //
		app.ble = new BLE();

        app.ble.on('bluetoothOn', () => { app.log('BLE ON') })
		app.ble.on('bluetoothOff', () => { app.log('BLE OFF') })

        app.ble.on('scanOn', () => {
			app.log('SCAN ON')

            // empty and update scan list
            $('#scanlist > ons-list-header').show();
            $('#scanlist > ons-list-header').html('Keresés...');
            $('#scanlist-refresh-button')[0].hide();
            $('#scanlist > ons-progress-bar').show();
            $('#scanlist > ons-list-item').remove();

		})
		app.ble.on('scanOff', () => {
			app.log('SCAN OFF')

            $('#scanlist-refresh-button')[0].show();
            $('#scanlist > ons-progress-bar').hide();

            if ($('#scanlist > ons-list-item').length == 0) {
                $('#scanlist > ons-list-header').html('Nem található okostáska a közelben');
            } else {
                $('#scanlist > ons-list-header').hide();
            }


		})
		app.ble.on('deviceFound', (device) => {
			app.log('Device found:', device)

			// Add this device to the pairable devices list
			$('#scanlist').append(`
                <ons-list-item tappable onclick="app.addSMB('${device.name}','${device.address}');">
                    <div class="left">
                        <ons-icon size="32px" icon="fa-briefcase"></ons-icon>
                    </div>
                    <div class="center">
                        <span class="list-item__title">${device.name}</span>
                        <span class="list-item__subtitle">${device.address}</span>
                    </div>
                </ons-list-item>
			`);
		})
        // ----------------------------  //

        // SMB //

        app.smbList = new SMBList(app.ble);

        console.log('SMBList created')

        app.smbList.on('smb-added', (smb) => {

            app.log('SMB added:',smb);
            app.db.setItem('smblist',JSON.stringify(app.smbList.serialize()));

            smb.on('ble-state-change', (state) => {
                $('#smb-list-item-'+smb.serial_nr+' span.ble-state').html(`<ons-icon size="16px" class="state_${state}" icon="fa-bluetooth"></ons-icon>`);
                if ($('#bag-settings-connection-bt').length>0) $('#bag-settings-connection-bt').html(app.connectionstate[state]);
            });

            smb.on('ppp-state-change', (state) => {
                $('#smb-list-item-'+smb.serial_nr+' span.ppp-state').html(`<ons-icon size="16px" class="state_${state}" icon="fa-globe"></ons-icon>`);
                if ($('#bag-settings-connection-tunnel').length>0) $('#bag-settings-connection-tunnel').html(app.connectionstate[state]);
            });

            smb.on('online-state-change', (online) => {
                if (!online) {
                    app.showNotification('Az SMB offline állapotba került.');
                }
            });

            smb.on('beacon-update', (address) => {
                app.log('beacon updated: ',address);

                // check if we need to notify


                // TODO: handle beacons separately
                if (document.getElementById('myNav').topPage.id == 'page-bag-beacon') app.renderSMBBeacons();
                if (document.getElementById('myNav').topPage.id == 'page-bag-unlinked') app.renderSMBUnlinkedBeacons();

            });

            smb.on('smb-mutated', () => {
                app.db.setItem('smblist',JSON.stringify(app.smbList.serialize()));
            });

            smb.on('auth-fail', () => {
                ons.notification.toast('Sikertelen azonosítás: SMN-'+smb.serial_nr, { timeout: 2000 });

                /*
                app.db.setItem('smblist',JSON.stringify(app.smbList.serialize()));

                if (document.getElementById('myNav').topPage.id == 'page-baglist') {
                    app.renderSMBList();
                }
                */

                if ( $('#dialog-bag-pin').length>0 && ('_adding' in $('#dialog-bag-pin')[0]) && $('#dialog-bag-pin')[0]._adding && $('#dialog-bag-pin')[0]._tmp_address == smb.ble_address ) {

                    $('#modal-loader').hide();
                    delete $('#dialog-bag-pin')[0]._adding;
                    $('#dialog-bag-pin')[0]._tmp_address = null;
                    $('#dialog-bag-pin')[0]._tmp_name = null;
                }

            });

            smb.on('auth-success', () => {

                if ( $('#dialog-bag-pin').length>0 && ('_adding' in $('#dialog-bag-pin')[0]) && $('#dialog-bag-pin')[0]._adding && $('#dialog-bag-pin')[0]._tmp_address == smb.ble_address ) {
                    $('#modal-loader').hide();

                    var sn;
                    app.smbList.forEach( (item, i) => {
                        if (item.ble_address == $('#dialog-bag-pin')[0]._tmp_address) sn = item.serial_nr;
                    });

                    app.db.setItem('currentsmb',JSON.stringify({sn: sn, addr: $('#dialog-bag-pin')[0]._tmp_address}));

                    delete $('#dialog-bag-pin')[0]._adding;
                    $('#dialog-bag-pin')[0]._tmp_address = null;
                    $('#dialog-bag-pin')[0]._tmp_name = null;
                }

            });

            smb.on('battery-charge', (percentage) => {

                console.log('battery',percentage);

                var idx = Math.floor(percentage / 20);
                $('.batterystate').html(app.batterystate[idx]);
            });

            smb.on('location', (location) => {

                console.log('Location updated',location);
                console.log(smb.locationMarker);

                app.db.setItem('smblist',JSON.stringify(app.smbList.serialize()));
                if ( ('locationMarker' in smb) && smb.locationMarker!=undefined ) smb.locationMarker.setPosition({ lat: parseFloat(location.lat.replace(',','.')), lng: parseFloat(location.lon.replace(',','.')) });

            });

            smb.on('hotspot-set', () => {
                $('#modal-loader').hide();
                console.log('hotspot set');
            });

            smb.on('hotspot-state', (result) => {

                console.log('hotspot state',result);

                app.db.setItem('smblist',JSON.stringify(app.smbList.serialize()));

                if ($('#bag-settings-hotspot').length>0) {
                    $('#bag-settings-hotspot')[0].checked = result.enabled;
                    $('#bag-settings-ssid').val(result.ssid);
                    $('#bag-settings-password').val(result.password);
                }
            });

            app.renderSMBList();

        });

        app.smbList.on('smb-removed', (smb) => {

            app.log('SMB removed:',smb);

            var currentsmb = app.currentSMB();
            if (currentsmb && currentsmb.sn == smb.serial_nr && currentsmb.addr == smb.ble_address) {
                app.db.removeItem('currentsmb');
                app.log('removed current smb');

                /* select next SMB if any */
                let nextsmb = app.smbList.getAt(0);
                if (nextsmb) app.currentSMB({ sn: nextsmb.serial_nr, addr: ble_address });
            }

            // delete the associated name of the SMB if exists
            let bagname = app.db.getItem('smbnames');
            if (bagname) {
                try {
                    bagname = JSON.parse(bagname);
                } catch(err) {
                    app.log(err);
                }
                if (bagname) {
                    if (serial in bagname) delete bagname[serial];
                    try {
                        bagname = JSON.stringify(bagname);
                    } catch(err) {
                        app.log(err);
                        return;
                    }
                    app.db.setItem('smbnames',bagname);
                }
            }

            app.db.setItem('smblist',JSON.stringify(app.smbList.serialize()));
            console.log('removed: ',app.smbList);

            fn.load('page-baglist.html',null,true);

            app.renderSMBList();

        });

        var _t = app.db.getItem('smblist');
        if (_t) {
            app.smbList.restore(app.ble, JSON.parse(_t));
        }


        /* FIREBASE */

        FCMPlugin.onTokenRefresh( async (token) => {

            console.log('FCM Token refresh:', token)

            if (token!=null) {
                // save token
                app.db.setItem('fcmtoken',token);

                // send token to gateway
                try {
                    await REST.sendToken( token )
                } catch(err) {
                    console.warn(err)
                }

            }
        });

        FCMPlugin.getToken( async (token) => {

            console.log('FCM Get token: ', token)

            if (token!=null) {

                // save token
                app.db.setItem('fcmtoken',token);

                // send token to gateway
                try {
                    await REST.sendToken( token )
                } catch(err) {
                    console.warn(err)
                }

            }
        }, (err) => { console.log(err); } );

        FCMPlugin.onNotification(

            // Notification callback
            (data) => {

                //console.log(data);

                if(data.wasTapped){
                    //Notification was received on device tray and tapped by the user.

                } else {
                    //Notification was received in foreground. Maybe the user needs to be notified.
                    navigator.notification.beep(1);

                }
            },

            // Success callback
            (msg) => {
            },

            // Error callback
            (err) => {
            }

        );

        /*
        app.pingTimer = setInterval( () => {

            app.smbList.forEach((item) => {

                if (item.online_state) {
                    // ping
                    item.ping( (err, duration) => {
                        if (err) {
                            console.log('Ping error',item.serial_nr,err)
                            return
                        }
                        app.smbLocalData( item.serial_nr, {lastseen : Date.now() } )
                    } )
                } else {
                    // offline state
                }

            });

        }, 15000)
        */

        //app.renderSMBList();
        // ----------------------------  //

    },

    onMapInit: function(map) {

        console.log('Initializing map')

        app.smbList.forEach( ( smb ) => {

            var locObj = new plugin.google.maps.LatLng(smb.location.lat, smb.location.lon);

            smb.locationMarker = map.addMarker({
                   position: locObj,
                   title: smb.serial_nr,
                   icon: 'img/bag-white-64.png'
            }, function(marker) {
                 // marker.showInfoWindow(); // Show infowindow
            });


        });

    },

    onOnsenInit: function(event) {

        if (event.target.tagName.toLowerCase()!='ons-page') return;
        var page = event.target;

        /* Onsen page inits */
        app.log('Page init',page);

        /* no current SMB -> navigate to smblist */
        if (page.id == 'page-bag-beacon') {
            document.getElementById('myNav').replacePage('page-baglist.html').then( () => {
                var menu = document.getElementById('mainmenu')
                if (menu) menu.close();
            });
        }
    },

    onOnsenShow: function(event) {
        if (event.target.tagName.toLowerCase()!='ons-page') return;
        var page = event.target;
        /* Onsen page shows */

        app.log('Page show', page)

        if (page.id == 'page-scan') {
            app.ble.startScan(5000);
        }

        if (page.id == 'page-baglist' || page.id == 'page-home') {
            // console.log('baglist-show');
            app.renderSMBList();
        }

        if (page.id == 'page-bag-settings') {
            app.renderSMBSettings();
        }

        if (page.id == 'page-bag-beacon') {
            app.renderSMBBeacons();
        }

        if (page.id == 'page-bag-beacon-unlinked') {
            app.renderSMBUnlinkedBeacons();
        }

        if (page.id == 'page-bag-map') {

            console.log('generating map');

            /* initialize the map */
//            app.map = plugin.google.maps.Map.getMap(document.getElementById('smb-map'));

            app.map = plugin.google.maps.Map.getMap(document.getElementById('smb-map'), {
                'mapType': plugin.google.maps.MapTypeId.ROADMAP,
                'controls': {
                    'compass': true,
                    'indoorPicker': false,
                    'myLocationButton': false,
                    'myLocation': true,   // (blue dot)
                    'zoom': false,          // android only
                    'mapToolbar': false     // android only
                },
                'gestures': {
                    'scroll': true,
                    'tilt': true,
                    'rotate': true,
                    'zoom': true
                },

                'camera' : {
                    zoom: 8
                }
            });

            // The MAP_READY event notifies the native map view is fully ready to use.
            app.map.on(plugin.google.maps.event.MAP_READY, app.onMapInit);

            $('#map-location-buttons > .smb-location-button').remove();

            var html = '';
            app.smbList.forEach( (smb) => {

                console.log(smb)
                let ldata = app.smbLocalData(smb.serial_nr);

                let color = 'black';
                if (ldata && ('color' in ldata)) color = ldata.color;

                html += `<ons-speed-dial-item class="smb-location-button" onclick="app.goToBagPosition('${smb.serial_nr}','${smb.ble_address}');">
                            <ons-icon icon="fa-briefcase" style="color: ${color}"></ons-icon>
                        </ons-speed-dial-item>`;
            });

            if (html!='') $('#map-location-buttons').append(html);

        }

    },

    onOnsenHide: function(event) {
        if (event.target.tagName.toLowerCase()!='ons-page') return;
        var page = event.target;
        /* Onsen page hides */

        if (page.id == 'page-scan.html') {
            app.ble.stopScan();
        }

    },

    /* METHODS */

    getAddress: async function(latLng) {

        return new Promise( (resolve, reject) => {

            plugin.google.maps.Geocoder.geocode({
                position: latLng
            }, (results) => {

                if (results.length === 0) {
                    reject('Ismeretlen cím');
                    return;
                }

                let address = [
                    results[0].subThoroughfare || "",
                    results[0].thoroughfare || "",
                    results[0].locality || "",
                    results[0].adminArea || "",
                    results[0].postalCode || "",
                    results[0].country || ""].join(", ");

                resolve(address,results);

            });


        });

    },

    setHotspot: async function(enabled,ssid,password) {
        let currentsmb = app.currentSMB();
        if (currentsmb) {
            $('#modal-loader').show();

            let smb = app.smbList.get(currentsmb.sn,currentsmb.addr);
            if (!smb) {
                // Invalid SMB
                app.log('Invalid SMB at address '+cursmb.addr+' with serial '+cursmb.sn);
                $('#modal-loader').hide();
                ons.notification.toast('Nincs aktív kapcsolat a táskával.', { timeout: 2000 });
                return false;
            }

            if (!(smb.online_state && ( (smb.ppp_state=='authorized' || smb.ble_state=='authorized') ) ) ) {
                ons.notification.toast('A táska nem érhető el', { timeout: 2000 });
                return false;
            }

            try {
                await smb.setHotspot(enabled,ssid,password);
            } catch(err) {
                console.log(err);
                $('#bag-settings-hotspot')[0].checked = !enabled;
                $('#modal-loader').hide();
                if (enabled) ons.notification.toast('Az internetmegosztás bekapcsolása sikertelen.', { timeout: 2000 });
                else ons.notification.toast('Az internetmegosztás kikapcsolása sikertelen.', { timeout: 2000 });

                $('#bag-settings-ssid').prop('disabled', $('#bag-settings-hotspot')[0].checked)
                $('#bag-settings-password').prop('disabled', $('#bag-settings-hotspot')[0].checked)

                return
            }

            if (enabled) ons.notification.toast('Internetmegosztás bekapcsolva.', { timeout: 2000 });
            else ons.notification.toast('Internetmegosztás kikapcsolva.', { timeout: 2000 });

            $('#bag-settings-ssid').prop('disabled', $('#bag-settings-hotspot')[0].checked)
            $('#bag-settings-password').prop('disabled', $('#bag-settings-hotspot')[0].checked)

        } else {
            ons.notification.toast('Nincs kiválasztott táska', { timeout: 2000 });
            return false;
        }

    },

    setHotspotPassword: function() {

        var newpass = $('#bag-settings-password').val();

        if (newpass.length<8) {
            ons.notification.toast('A jelszónak legalább 8 karakteresnek kell lennie', { timeout: 2000 });
            $('#bag-settings-password').val($('#bag-settings-password')[0].focusedValue);
            return
        }

        app.hotspotPasswordDialog();

    },


    setHotspotSSID: function() {

        var newssid = $('#bag-settings-ssid').val();

        if (newssid.length<3) {
            ons.notification.toast('Az SSID hossza legalább 8 karakteres kell hogy lennie', { timeout: 2000 });
            $('#bag-settings-ssid').val($('#bag-settings-ssid')[0].focusedValue);
            return
        }

        app.hotspotSSIDDialog();

    },

    hotspotPasswordDialog: async function() {

        var dialog = document.getElementById('dialog-bag-set-hotspot-password');

        if (!dialog) {
            await ons.createElement('dialog-bag-set-hotspot-password.html', { append: true })
        }

        $('#dialog-bag-set-hotspot-password')[0].show();

    },


    hotspotSSIDDialog: async function() {

        var dialog = document.getElementById('dialog-bag-set-hotspot-ssid');

        if (!dialog) {
            await ons.createElement('dialog-bag-set-hotspot-ssid.html', { append: true })
        }

        $('#dialog-bag-set-hotspot-ssid')[0].show();

    },

    clientKeyDialog: async function() {

        var dialog = document.getElementById('dialog-bag-renew-clientkey');

        if (!dialog) {
            await ons.createElement('dialog-bag-renew-clientkey.html', { append: true })
        }

        $('#dialog-bag-renew-clientkey')[0].show();

    },

    resetClientKey: async function() {

        var currentsmb = app.currentSMB();

        if (!currentsmb) {
            ons.notification.toast('Nincs kiválasztott táska', { timeout: 2000 });
            return false;
        }

        var smb = app.smbList.get(currentsmb.sn,currentsmb.addr);
        if (smb) {
            if (smb.online_state && ( (smb.ppp_state=='authorized' || smb.ble_state=='authorized') ) ) {
                $('#modal-loader').show();
            } else {
                ons.notification.toast('Nincs aktív kapcsolat a táskával.', { timeout: 2000 });
                return false;
            }
        } else {
            ons.notification.toast('A táska nem érhető el', { timeout: 2000 });
            return false;
        }

        var client_key = '';

        try {
            client_key = await smb.resetClientKey();
        } catch(err) {
            console.warn(err)
            client_key = ''
        }

        if (client_key == null || client_key == '') {
            ons.notification.toast('Sikertelen kód kérés', { timeout: 2000 });
            $('#modal-loader').hide();
            return
        }

        REST.authcode = client_key;

        if ($('#bag-settings-webid').length>0) $('#bag-settings-webid').html(client_key)

        $('#modal-loader').hide();

    },

    goToHomePosition: async function() {
        let home = app.homeLocation();

        console.log('home',home);

        if (!home) {
            var dialog = document.getElementById('dialog-home-position');

            if (!dialog) {
                await ons.createElement('dialog-home-position.html', { append: true })
            }

            $('#dialog-home-position')[0].show();

            plugin.google.maps.LocationService.getMyLocation({ /* enableHighAccuracy: true */ }, async (location) => {
                console.log('Location received',location);
                $('#currentLocationAddress').location = location.latLng;

                try {
                    var myaddress = await app.getAddress(location.latLng);
                } catch(err) {
                    app.log(err);
                    myaddress = err;
                }

                $('#currentLocationAddress').html(myaddress);

            }, (error) => {
                app.log(error);
            });

            return;
        }

        app.map.animateCamera({
            target: home,
            zoom: 15,
            duration: 3000
        });

    },

    goToBagPosition: async function(sn,addr) {

        let smb = app.smbList.get(sn,addr);

        if ( smb && ('location' in smb) && smb.location ) {

            app.map.animateCamera({
                target: new plugin.google.maps.LatLng( parseFloat(smb.location.lat.replace(',','.')), parseFloat(smb.location.lon.replace(',','.')) ),
                zoom: 15,
                duration: 1000
            });

        }

    },

    goToCurrentPosition: async function() {

        var permission = await plugin.google.maps.LocationService.hasPermission();

        if (permission) {

            plugin.google.maps.LocationService.getMyLocation({ /* enableHighAccuracy: true */ }, (location) => {
                console.log('Location received',location);
                app.map.animateCamera({
                    target: location.latLng,
                    zoom: 15,
                    duration: 3000
                });
            }, (error) => {
                app.log(error);
            });

        }

    },

    isCurrentSMBConnected: function() {
        var currentsmb = app.currentSMB();

        if (!currentsmb) return false;

        return app.smbList.get(currentsmb.sn,currentsmb.addr).online_state;
    },

    smbLocalData: function(sn,data) {

        if (!sn) return false;

        var localdata = app.db.getItem('smblocdata');

        if (localdata) {
            try {
                localdata = JSON.parse(localdata);
            } catch(err) {
                localdata = {};
            }
            if (!localdata) {
                localdata = {}
            }
        } else {
            localdata = {}
        }

        if (!data) {
            // get data //
            if (sn in localdata) return localdata[sn];
            else return null;
        }

        // set data //
        var cdata;
        if (sn in localdata) cdata = localdata[sn];
        else cdata = {
            name: 'SMB-'+sn,
            sn: sn,
            color: 'white',
            lastseen: 0,
            lastgps: null
        }

        cdata = Object.assign( cdata, data );
        localdata[sn] = cdata;

        try {
            localdata = JSON.stringify(localdata);
        } catch(err) {
            app.log(err);
            return false;
        }

        app.db.setItem('smblocdata',localdata);

        return cdata;

    },

    homeLocation: function(location) {

        if (location == null) {
            // get home location //
            let home = app.db.getItem('home');
            if (home) {
                try {
                    home = JSON.parse(home);
                } catch(err) {
                    app.log(err);
                    return null;
                }
                if (!home) return null;
                return new plugin.google.maps.LatLng(home.lat,home.lng);
            }
            return null;
        }

        // set home location //
        let home = { lat: location.lat, lng: location.lng }
        try {
            home = JSON.stringify(home)
        } catch(err) {
            app.log(err);
            return;
        }

        app.db.setItem('home',home);

        return true;

    },

    currentSMB: function(smb) {

        if (smb) {
            // set SMB
            app.db.setItem('currentsmb',JSON.stringify({sn: smb.sn, addr: smb.addr}));
            if ( ('name' in smb) || ('color' in smb) ) {

                var localdata = app.smbLocalData(smb.sn);

                if ('name' in smb) localdata.name = smb.name;
                if ('color' in smb) localdata.color = smb.color;

                app.smbLocalData(smb.sn,localdata);

            }
            return;
        }

        // get SMB
        var currentbag = app.db.getItem('currentsmb');
        if (currentbag) {
            try {
                currentbag = JSON.parse(currentbag);
            } catch (err) {
                app.log(err);
                return null;
            }
            if (currentbag) {
                currentbag.name = 'SMB-'+currentbag.sn;
                currentbag.color = 'white';

                // get bag data associated for this SMB
                var localdata = app.smbLocalData(currentbag.sn);
                if (localdata) {
                    if ('name' in localdata) currentbag.name = localdata.name;
                    if ('color' in localdata) currentbag.color = localdata.color;
                }

                return currentbag;
            }
        }

        return null;
    },

    addSMB: async function(name,address,pin) {

        var dialog = document.getElementById('dialog-bag-pin');

        if (!dialog) {
            await ons.createElement('dialog-bag-pin.html', { append: true })
        }

        if (pin == null) {
            // request PIN Code //
            $('#dialog-bag-pin')[0]._tmp_name = name;
            $('#dialog-bag-pin')[0]._tmp_address = address;

            $('#dialog-bag-pincode').val('');

            $('#dialog-bag-pin')[0].show();

        } else {

            $('#dialog-bag-pin')[0].hide();

            $('#dialog-bag-pin')[0]._adding = true;

            $('#modal-loader').show();

            app.smbList.add($('#dialog-bag-pin')[0]._tmp_name,$('#dialog-bag-pin')[0]._tmp_address,pin);
            app.log('Adding SMB: ',$('#dialog-bag-pin')[0]._tmp_name,$('#dialog-bag-pin')[0]._tmp_address,pin);

            $('#myNav')[0].popPage();

        }
    },

    selectSMB: function(serial,address) {

        let currentbag = app.currentSMB();
        if (currentbag) {
            // the tapped SMB is same as the current SMB
            if (currentbag.sn == serial && currentbag.addr == address) return;
        }

        app.db.setItem('currentsmb',JSON.stringify({sn: serial, addr: address}));
        app.renderSMBList();

        //currentbag = app.currentSMB();
        //REST.authcode ( currentbag.)

        ons.notification.toast('Táska kiválasztva', { timeout: 2000 });

        let smb = app.smbList.get(currentbag.sn,currentbag.addr)
        if (smb) {

            REST.sendToken( smb.client_key ).then( () => { console.log('Token sent') } ).catch( (err) => { console.warn(err) })

        }

        //$('#myNav')[0].replacePage('page-bag-beacon.html');
        //app.renderSMBBeacons();

    },

    removeSMB: function(serial,address) {

        if (serial == null && address == null) {
            let _smb = app.currentSMB();
            if (_smb) {
                serial = _smb.sn;
                address = _smb.addr;
            } else return;
        }

        // TODO: handle if the selected SMB is the current SMB
        app.log('Trying to remove SMB with serial '+serial+' on address '+address);
        app.smbList.remove(serial,address);

    },

    setSMBNameQuery: function(serial, name) {

        let result = app.setSMBName(serial, name);

        if (result) {
            $('#bag-settings-name').html(name)
        } else {
            $('#bag-settings-name').html(app.getSMBName(serial))
        }

    },

    setSMBName: function(serial,name) {
        if (serial == null) {
            let _currentsmb = app.currentSMB();
            if (_currentsmb) serial = _currentsmb.sn;
            else return false;
        }

        if (name == null || name == '') return false;

        app.smbLocalData(serial,{ name: name });

        return true;

    },

    getSMBName: function(serial) {

    },

    setSMBColor: function(serial,color) {
        if (serial == null) {
            let _currentsmb = app.currentSMB();
            if (_currentsmb) serial = _currentsmb.sn;
            else return false;
        }

        if (name == null || name == '') return false;

        app.smbLocalData(serial,{ color: color });

        return true;

    },

    showNotification: function(text,title) {
        if (title == null) title = 'SMB';

        cordova.plugins.notification.local.schedule({
            id: app._nextNotificationId(),
            title: title,
            text: text,
            foreground: true
        });

    },

    _nextNotificationId: function() {
        app._nid++;
        return app._nid;
    },

    renderSMBSettings: function() {
        let currentsmb = app.currentSMB();
        if (currentsmb) {
            $('#bag-settings-bagid').html('SMB-'+currentsmb.sn);
            $('#bag-settings-name').val(currentsmb.name);

            let smb = app.smbList.get(currentsmb.sn,currentsmb.addr);
            if (!smb) {
                // Invalid SMB
                app.log('Invalid SMB at address '+cursmb.addr+' with serial '+cursmb.sn);
                return
            }

            $('#bag-settings-webid').html(smb.client_key);

            $('#bag-settings-hotspot')[0].checked = smb.hotspot.enabled;

            $('#bag-settings-ssid').val( smb.hotspot.ssid );
            $('#bag-settings-password').val(smb.hotspot.password);

            $('#bag-settings-ssid').prop('disabled',smb.hotspot.enabled)
            $('#bag-settings-password').prop('disabled',smb.hotspot.enabled)

            $('#bag-settings-connection-bt').html(app.connectionstate[smb.ble_state])
            $('#bag-settings-connection-tunnel').html(app.connectionstate[smb.ppp_state])

        } else {
            $('#bag-settings-bagid').html('ISMERETLEN');
            $('#bag-settings-name').val('');
        }

    },

    renderSMBList: function() {

        var baglist = $('#page-baglist');

        if (baglist && baglist.length>0) {

            $('#baglist > ons-list-item').remove();

            if (app.smbList.length() == 0) {
                $('#baglist').append(`
                    <ons-list-item>Még nincs okostáska párosítva az applikációhoz.<br/>Kattintson a jobb alsó saroban lévő + ikonra táskák kereséséhez.</ons-list-item>
                `);
                return;
            }

            app.smbList.forEach( (smb) => {

                let _currentsmb = app.currentSMB();
                let _currentclass = '';
                let _smbname = '';
                if (_currentsmb && _currentsmb.sn == smb.serial_nr) _currentclass = 'selected';

                let localdata = app.smbLocalData(smb.serial_nr)

                if (localdata) _smbname = localdata.name;
                else _smbname = 'SMB-'+smb.serial_nr;

                $('#baglist').append(`
                    <ons-list-item id="smb-list-item-${smb.serial_nr}" class="${_currentclass}" tappable onclick="app.selectSMB('${smb.serial_nr}','${smb.ble_address}');">
                        <div class="left">
                            <ons-icon class="bagicon" size="32px" icon="fa-briefcase"></ons-icon>
                        </div>
                        <div class="center">
                            <span class="list-item__title">${_smbname}</span>
                            <span class="list-item__subtitle"><span class="ble-state"><ons-icon size="16px" class="state_${smb.ble_state}" icon="fa-bluetooth"></ons-icon></span> <span class="ppp-state"><ons-icon size="16px" class="state_${smb.ppp_state}" icon="fa-globe"></ons-icon></span></span>
                        </div>
                        <div class="right">
                            <span class="list-item__title batterystate">
                                <span class="fa-stack">
                                    <i class="fas fa-battery-empty fa-stack-1x"></i>
                                    <i class="fas fa-question fa-stack-1x" style="font-size: 0.25em;"></i>
                                </span>
                            </span>
                        </div>
                    </ons-list-item>
                `);

                if (smb.battery_charge!=null) {
                    let idx = Math.floor(smb.battery_charge / 20);
                    $('.batterystate').html(app.batterystate[idx]);
                }

            });

        }

    },

    renderSMBBeacons: function() {

        let smbkey = null;
        let cursmb = app.currentSMB();

        if (!cursmb) {
            // No SMB selected
            app.log('Rendering SMB Beacon list, but no SMB is selected');
            return
        }

        let smb;
        if ( ('smbList' in app) && app.smbList ) smb = app.smbList.get(cursmb.sn,cursmb.addr);

        if (!smb) {
            // Invalid SMB
            app.log('Invalid SMB at address '+cursmb.addr+' with serial '+cursmb.sn);
            return
        }

        $('#smb-beaconlist').empty();

        if (smb.battery_charge!=null) {
            let idx = Math.floor(smb.battery_charge / 20);
            $('.batterystate').html(app.batterystate[idx]);
        }

        let _beaconkeys = Object.keys(smb.beacons);
        if (_beaconkeys.length == 0) {
            $('#smb-beaconlist').html('<ons-list-header>Nem találhatóak eszközök a táska közelében</ons-list-header>');
            app.log('No beacons found in SMB at address '+cursmb.addr+' with serial '+cursmb.sn);
        } else {

            var paired_in = '<ons-list-header id="smb-beacons-paired">Táskában van</ons-list-header>';
            var paired_out = '<ons-list-header id="smb-beacons-unpaired">Nincs a táskában</ons-list-header>';
            var unpaired = '<ons-list-header id="smb-beacons-unpaired">Talált eszközök</ons-list-header>';

            _beaconkeys.forEach( (key) => {
                let beacon = smb.beacons[key];

                let beaconid = key.split(':').join('');

                let beaconname = ( ('n' in beacon) && beacon.n!=null && beacon.n!='') ? beacon.n : 'Névtelen eszköz';
                let beacontype = ( ('t' in beacon) && beacon.t!=null && beacon.t!='') ? parseInt(beacon.t) : 0;

                if ( beacon.p && beacon.d < 2) {
                    paired_in+=`
                        <ons-list-item id="smb-beacon-${beaconid}">
                            <div class="left">
                                <ons-icon size="32px" icon="${app.beacon_icons[beacontype]}" style="color:${app.beacon_colors[beacon.c]};"></ons-icon>
                            </div>
                            <div class="center">
                                <span class="list-item__title">${beaconname}</span>
                                <span class="list-item__subtitle">${key}</span>
                            </div>
                            <div class="right">
                                <ons-toolbar-button onclick="app.showBeaconUpdateDialog('${key}');">
                                    <ons-icon icon="fa-cog"></ons-icon>
                                </ons-toolbar-button>
                                <ons-toolbar-button onclick="app.showBeaconUnlinkDialog('${key}')">
                                    <ons-icon icon="fa-unlink"></ons-icon>
                                </ons-toolbar-button>
                            </div>
                        </ons-list-item>
                        `;
                }

                if (beacon.p && beacon.d > 1) {
                    paired_out+=`
                        <ons-list-item id="smb-beacon-${beaconid}">
                            <div class="left">
                                <ons-icon size="32px" icon="${app.beacon_icons[beacontype]}" style="color:${app.beacon_colors[beacon.c]};"></ons-icon>
                            </div>
                            <div class="center">
                                <span class="list-item__title">${beaconname}</span>
                                <span class="list-item__subtitle">${key}</span>
                            </div>
                            <div class="right">

                            </div>
                        </ons-list-item>
                        `;
                }

                //console.log(key,beacon);
            });

            $('#smb-beaconlist').append(paired_in);
            $('#smb-beaconlist').append(paired_out);
        }

    },

    renderSMBUnlinkedBeacons: function() {

        let smbkey = null;
        let cursmb = app.currentSMB();

        if (!cursmb) {
            // No SMB selected
            app.log('Rendering SMB Beacon list, but no SMB is selected');
            return
        }

        let smb = app.smbList.get(cursmb.sn,cursmb.addr);
        if (!smb) {
            // Invalid SMB
            app.log('Invalid SMB at address '+cursmb.addr+' with serial '+cursmb.sn);
            return
        }

        $('#smb-unlinked-beaconlist').empty();

        let _beaconkeys = Object.keys(smb.beacons);
        if (_beaconkeys.length == 0) {
            $('#smb-unlinked-beaconlist').html('<ons-list-header>Nem találhatóak eszközök a táska közelében</ons-list-header>');
            app.log('No beacons found in SMB at address '+cursmb.addr+' with serial '+cursmb.sn);
        } else {

            $('#smb-unlinked-beaconlist').append('<ons-list-header id="smb-beacons-unpaired">Talált eszközök</ons-list-header>');

            _beaconkeys.forEach( (key) => {
                let beacon = smb.beacons[key];

                let beaconid = key.split(':').join('');

                if (!beacon.p && beacon.d < 2) {
                    $('#smb-unlinked-beaconlist').append(`
                        <ons-list-item id="smb-beacon-${beaconid}" tappable onclick="app.showBeaconLinkDialog('${key}');">
                            <div class="left">
                                <ons-icon size="32px" icon="fa-caret-up" style="color:${app.beacon_colors[beacon.c]};"></ons-icon>
                            </div>
                            <div class="center">
                                <span class="list-item__title">${key}</span>
                                <span class="list-item__subtitle">Érintse meg a hozzáadáshoz</span>
                            </div>
                        </ons-list-item>
                        `);
                }

            });

        }

    },

    linkBeacon: async function(smbserial,smbaddress,beaconaddr,beaconname,beacontype=0,essential=false) {

        if (smbserial == null) {
            var currentsmb = app.currentSMB();
            if (currentsmb) {
                smbserial = currentsmb.sn;
                smbaddress = currentsmb.addr;
            } else {
                return false;
            }
        }

        $('#modal-loader').show();

        var success = true;
        try {
            await app.smbList.get(smbserial,smbaddress).linkBeacon(beaconaddr,beaconname,beacontype,essential);
        } catch (err) {
            app.log(err);
            success = false;
        }

        await $('#dialog-bag-linkbeacon')[0].hide();

        fn.pop()

        $('#modal-loader').hide();

        if (success) {
            ons.notification.toast('Sikeres összekapcsolás', { timeout: 3000 });
        } else {
            ons.notification.toast('Sikertelen összekapcsolás', { timeout: 3000 });
        }

    },

    selectEssentialBeacon: function(element) {
        $('.dialog-bag-iconlist ons-icon').removeClass('selected');
        $(element).addClass('selected');
    },

    updateBeacon: async function(smbserial,smbaddress,beaconaddr,beaconname,beacontype=0,essential=false) {

        if (smbserial == null) {
            var currentsmb = app.currentSMB();
            if (currentsmb) {
                smbserial = currentsmb.sn;
                smbaddress = currentsmb.addr;
            } else {
                return false;
            }
        }

        $('#modal-loader').show();

        var success = true;
        try {
            await app.smbList.get(smbserial,smbaddress).updateBeacon(beaconaddr,beaconname,beacontype,essential);
        } catch (err) {
            app.log(err);
            success = false;
        }

        $('#modal-loader').hide();

        app.renderSMBBeacons();

        if (!success) {
            ons.notification.toast('Sikertelen módosítás', { timeout: 3000 });
        }

    },

    unlinkBeacon: async function(smbserial,smbaddress,beaconaddr) {

        if (smbserial == null) {
            var currentsmb = app.currentSMB();
            if (currentsmb) {
                smbserial = currentsmb.sn;
                smbaddress = currentsmb.addr;
            } else {
                return false;
            }
        }

        $('#modal-loader').show();

        var success = true;
        try {
            await app.smbList.get(smbserial,smbaddress).unlinkBeacon(beaconaddr);
        } catch (err) {
            app.log(err);
            success = false;
        }

        $('#modal-loader').hide();

        app.renderSMBBeacons();

        if (!success) {
            ons.notification.toast('Sikertelen leválasztás', { timeout: 3000 });
        }

    },

    // DIALOGS & MODALS
    confirmSMBRemove: function() {
        var dialog = document.getElementById('dialog-bag-remove-confirm');
        if (dialog) {
            dialog.show();
        } else {
            ons.createElement('dialog-bag-remove-confirm.html', { append: true })
            .then(function(dialog) {
                dialog.show();
            });
        }
    },

    showBeaconLinkDialog: function(beaconaddr) {

        let currentsmb = app.currentSMB();
        if (!currentsmb) {
            ons.notification.toast('Nincs kiválasztott táska', { timeout: 2000 });
            return
        }

        let smb = app.smbList.get(currentsmb.sn,currentsmb.addr);
        if (!smb) {
            // Invalid SMB
            ons.notification.toast('A táska nem érhető el', { timeout: 2000 });
            return
        }

        if (!(smb.online_state && ( (smb.ppp_state=='authorized' || smb.ble_state=='authorized') ) ) ) {
            ons.notification.toast('A táska nem érhető el', { timeout: 2000 });
            return
        }

        var dialog = document.getElementById('dialog-bag-linkbeacon');
        if (dialog) {
            $('#dialog-bag-linkbeacon')[0].beaconaddr = beaconaddr;
            $('#dialog-bag-linkbeacon-essential').prop('checked',false)
            $('#dialog-bag-linkbeacon-name').val('');
            $('#dialog-bag-linkbeacon-beaconlist ons-icon').removeClass('selected');
            $('#dialog-bag-linkbeacon-beaconlist ons-icon').first().addClass('selected');
            dialog.show();
        } else {
            ons.createElement('dialog-bag-linkbeacon.html', { append: true })
            .then(function(dialog) {
                $('#dialog-bag-linkbeacon')[0].beaconaddr = beaconaddr;
                $('#dialog-bag-linkbeacon-name').val('');
                dialog.show();
            });
        }
    },

    showBeaconUpdateDialog: function(beaconaddr) {

        let currentsmb = app.currentSMB();
        if (!currentsmb) {
            ons.notification.toast('Nincs kiválasztott táska', { timeout: 2000 });
            return
        }

        let smb = app.smbList.get(currentsmb.sn,currentsmb.addr);
        if (!smb) {
            // Invalid SMB
            ons.notification.toast('A táska nem érhető el', { timeout: 2000 });
            return
        }

        if (!(smb.online_state && ( (smb.ppp_state=='authorized' || smb.ble_state=='authorized') ) ) ) {
            ons.notification.toast('A táska nem érhető el', { timeout: 2000 });
            return
        }

        var beacon = smb.beacons[beaconaddr];

        var dialog = document.getElementById('dialog-bag-updatebeacon');
        if (dialog) {
            $('#dialog-bag-updatebeacon')[0].beaconaddr = beaconaddr;
            $('#dialog-bag-updatebeacon-essential').prop('checked',beacon.e)
            $('#dialog-bag-updatebeacon-name').val(beacon.n);
            $('#dialog-bag-updatebeacon-beaconlist ons-icon').removeClass('selected');
            $('#dialog-bag-updatebeacon-beaconlist ons-icon[data-beacon='+beacon.t+']').addClass('selected');
            dialog.show();
        } else {
            ons.createElement('dialog-bag-updatebeacon.html', { append: true })
            .then(function(dialog) {
                $('#dialog-bag-updatebeacon')[0].beaconaddr = beaconaddr;
                $('#dialog-bag-updatebeacon-essential').prop('checked',beacon.e)
                $('#dialog-bag-updatebeacon-name').val(beacon.n);
                $('#dialog-bag-updatebeacon-beaconlist ons-icon').removeClass('selected');
                $('#dialog-bag-updatebeacon-beaconlist ons-icon[data-beacon='+beacon.t+']').addClass('selected');
                dialog.show();
            });
        }
    },

    showBeaconUnlinkDialog: function(beaconaddr) {

        let currentsmb = app.currentSMB();
        if (!currentsmb) {
            ons.notification.toast('Nincs kiválasztott táska', { timeout: 2000 });
            return
        }

        let smb = app.smbList.get(currentsmb.sn,currentsmb.addr);
        if (!smb) {
            // Invalid SMB
            ons.notification.toast('A táska nem érhető el', { timeout: 2000 });
            return
        }

        if (!(smb.online_state && ( (smb.ppp_state=='authorized' || smb.ble_state=='authorized') ) ) ) {
            ons.notification.toast('A táska nem érhető el', { timeout: 2000 });
            return
        }

        var dialog = document.getElementById('dialog-bag-unlinkbeacon');
        if (dialog) {
            $('#dialog-bag-unlinkbeacon')[0].beaconaddr = beaconaddr;
            dialog.show();
        } else {
            ons.createElement('dialog-bag-unlinkbeacon.html', { append: true })
            .then(function(dialog) {
                $('#dialog-bag-unlinkbeacon')[0].beaconaddr = beaconaddr;
                dialog.show();
            });
        }

    }

}


$(function(){
    app.initialize();
});
