'use strict';

class BagUpdateDataModel {
	constructor(rawData) {
		this._defaultData();
		if (rawData) {
			this.parseData(rawData);
		}
	}

	parseData(rawData) {
		this._defaultData();
		this._setRecordIfPresent("ts", rawData);
		this._setRecordIfPresent("lon", rawData);
		this._setRecordIfPresent("lat", rawData);
		this._setRecordIfPresent("gps_ts", rawData);
		this._setRecordIfPresent("battery", rawData);
		this._setRecordIfPresent("phone_active", rawData);
		this._setRecordIfPresent("missing_gadgets", rawData);
		this._setRecordIfPresent("gadgets", rawData);
	}

	update(bdm) {
		let raw = bdm.getData();

		if (raw.lat === undefined || raw.lon === undefined || raw.lon === null || raw.lon === null || (raw.lon == 0 && raw.lat == 0)) {
			raw.lat = this.getLatitude();
			raw.lon = this.getLongitude();
			raw.gps_ts = this.getGPSTs();
		} else {
			raw.gps_ts = raw.ts || 0;
		}

		this.parseData(raw);
	}

	getGPSTs() {
		return this._record.gps_ts;
	}

	getBattery() {
		return this._record.battery;
	}

	getLongitude() {
		return this._record.lon;
	}

	getLatitude() {
		return this._record.lat;
	}

	isPhoneActive() {
		return this._record.phone_active;
	}

	getMissingGadgets() {
		return this._record.missing_gadgets;
	}

	getData() {
		return {...this._record};
	}

	_defaultData() {
		this._record = {
			ts : 0
		}
	}

	getTimeStamp() {
		return this._record["ts"] || 0;
	}

	validate() {
		if (Object.keys(this._record).length === 0) {
			return "Data is empty";
		}

		if (!this._record["ts"] || this._record["ts"] === 0) {
			return "Timestamp is mandatory, but not present";
		}

		return true;
	}

	_setRecordIfPresent(key, rawData) {
		if (rawData[key] !== undefined) {
			this._record[key] = rawData[key];
		}
	}
}

module.exports = BagUpdateDataModel;
