'use strict';
const CONST = require("../../constants.js");
const LOG   = require("../../core/Logger.js");

const DataAccessLayer = require("./DataAccessLayer.js");
const BagModel        = require("./BagModel.js");

const BAG_TABLE = "bagData";

class CachingDataAccessLayer extends DataAccessLayer {
	constructor(connector, cache) {
		super(connector);
		this._cache = cache;
	}

	getBagData(key) {
		let cached = this._cache.get(key, null);
		if (cached) return Promise.resolve(cached);
		return this._connector
			.getRecord(BAG_TABLE, key)
			.then(result => {
				if (result) {
					let bag = new BagModel(key, result);
					this._cache.add(key, bag);
					return bag;
				}

				return result;
			});
	}

	updateBagData(bagModel) {
		return this._connector
			.saveRecord(BAG_TABLE, bagModel.getData(), bagModel.getKey())
			.then(result => {
				this._cache.add(bagModel.getKey()	, bagModel);
				return result;
			});
	}

}

module.exports = CachingDataAccessLayer;