'use strict';

class NotificationDataModel {
	constructor(rawData) {
		this._defaultData();
		if (rawData) {
			this.parseData(rawData);
		}
	}

	parseData(rawData) {
		this._defaultData();
		this._setRecordIfPresent("os", rawData);
		this._setRecordIfPresent("key", rawData);
	}

	getKey() {
		return this._record.key;
	}

	getData() {
		return {...this._record};
	}

	_defaultData() {
		this._record = {
		}
	}

	validate() {
		if (Object.keys(this._record).length === 0) {
			return "Data is empty";
		}

		if (!this._record["os"]) {
			return "OS is mandatory, but not present";
		}

		if (!this._record["key"]) {
			return "Key is mandatory, but not present";
		}

		if (this._record["os"] !== "ios" && this._record["os"] !== "android") {
			return "OS must be either 'ios' or 'android'!";
		}

		return true;
	}

	_setRecordIfPresent(key, rawData) {
		if (rawData[key] !== undefined) {
			this._record[key] = rawData[key];
		}
	}
}

module.exports = NotificationDataModel;