'use strict';
const CONST = require("../../constants.js");
const LOG   = require("../../core/Logger.js");

const BagBasedHandlerBase = require("./BagBasedHandlerBase.js");
const HandlerException = require("./HandlerException.js");

class NotifyHandler extends BagBasedHandlerBase {
	static getRoute() {
		return "/:bagId/notify";
	}

	async _processGET() {
		return this.respondMethodNotAllowed("GET");
	}

	async _processPUT() {
		return this.respondMethodNotAllowed("PUT");
	}

	async _processPOST() {
		let notificationData = this._bag.getNotificationData();

		if (!notificationData) return this.respondNotFound("User has not set notification data!");
		if (!this._body || !this._body.data || !this._body.data.title || !this._body.data.message) {
			return this.respondBadRequest("Missing data fields!");
		}

		return this._firebase.messaging().sendToDevice(notificationData.getKey(), {
					notification: {
						title : this._body.data.title,
						body  : this._body.data.message
					},
					data : {
						bagId : this._bag.getKey()
					}
				})
				.then(r => {
					if (r.failureCount > 0) {
						return this.respondServiceUnavailable("Error during sending the message.");
					}
					return this.respondOk();
				});
	}

	async _processDELETE() {
		return this.respondMethodNotAllowed("DELETE");
	}
}

module.exports = NotifyHandler;