'use strict';
const CONST = require("../../constants.js");
const LOG   = require("../../core/Logger.js");

const BagBasedHandlerBase = require("./BagBasedHandlerBase.js");
const HandlerException = require("./HandlerException.js");

const NotificationDataModel = require("../dal/NotificationDataModel.js");

class NotifyRegisterHandler extends BagBasedHandlerBase {
	static getRoute() {
		return "/:bagId/notify/register";
	}

	async _processGET() {
		return this.respondMethodNotAllowed("GET");
	}

	async _processPUT() {
		let notificationData = new NotificationDataModel(this._body.data);
		let valid = notificationData.validate();

		if (valid !== true) return this.respondBadRequest(valid);

		this._bag.setNotificationData(notificationData);

		return this._saveBagData()
					.then(r => this.respondOk());
	}

	needsMasterAccess() {
		return false;
	}

	async _processPOST() {
		return this.respondMethodNotAllowed("POST");
	}

	async _processDELETE() {
		return this.respondMethodNotAllowed("DELETE");
	}
}

module.exports = NotifyRegisterHandler;