'use strict';
const ClusteringBase = require("./ClusteringBase.js");

class MeanShift extends ClusteringBase {
	buildClusters(range, epsilon) {
		let labels = [];
		let epower = epsilon * epsilon;
		let rpower = range * range;
		for (let i = 0; i < this._records.length; i++) {
			labels[i] = 0;
		}

		let means = [];
		let clusters = [];
		for (let i = 0; i < this._records.length; i++) {
			means.push([...this._records[i]]);
			clusters[i] = [i];
		}

		let iterations = 40;
		let updates  = 1
		while (iterations > 0 && updates > 0) {
			updates  = 0;

			for (let m = 0; m < means.length; m++) {
				let mean = means[m];
				if (mean === null) continue;
				let newMean = null;
				let items   = 0;
				for (let i = 0; i < this._records.length; i++) {
					let record = this._records[i];
					if (this.distanceSq(mean, record) < rpower) {
						items++;
						if (newMean) {
							newMean.forEach((v, idx) => {
								newMean[idx] = v + record[idx];
							})
						} else {
							newMean = [...record];
						}
					}
				}

				newMean.forEach((v, idx) => {
					newMean[idx] = v / items;
				});

				if (this.distanceSq(newMean, mean) > epower) {
					updates++;
				}
				means[m] = newMean;
			}

			for (let m = 0; m < means.length - 1; m++) {
				let mean = means[m];
				if (mean === null) continue;
				for (let j = m + 1; j < means.length; j++) {
					let omean = means[j];
					if (omean === null) continue;
					if (this.distanceSq(omean, mean) <= epower) {
						means[j] = null;
						clusters[m] = [...(clusters[m]), ...(clusters[j])];
						clusters[j] = null;
					}
				}
			}

			iterations--;
			console.error("Iteration", iterations, "updates", updates)
		}


		for (let i = 0; i < clusters.length; i++) {
			let cluster = clusters[i];
			if (cluster) {
				cluster.forEach( label => labels[label] = i);
			}
		}

		return labels;
	}

	distanceSq(v1, v2) {
		let ret = 0;
		for (let i = 0; i < v1.length; i++) {
			ret += (v1[i] - v2[i]) * (v1[i] - v2[i]);
		}
		return ret;
	}
}

module.exports = MeanShift;