'use strict';
const SecUtil               = require("../../core/SecUtil.js");
const BagUpdateDataModel    = require("./BagUpdateDataModel.js");
const NotificationDataModel = require("./NotificationDataModel.js");
const AlertDataModel        = require("./AlertDataModel.js");

class BagModel {
	constructor(key, rawData) {
		this._defaultData();
		this._key = key;
		if (rawData) {
			if (rawData._id !== key) {
				throw new Error(
					"Key doesn't match raw data's key '" + key + "' + vs '" + rawData._id + "'"
				)
			}
			this.parseData(rawData);
		}
	}

	parseData(rawData) {
		this._masterKey = rawData.masterKey;
		this._clientKey = rawData.clientKey;

		this._lastData  = null;
		if (rawData.lastData) {
			this.setLastData(new BagUpdateDataModel(rawData.lastData));
		}

		this._notificationData  = null;
		if (rawData.notificationData) {
			this.setNotificationData(new NotificationDataModel(rawData.notificationData));
		}

		this._alertData  = null;
		if (rawData.alertData) {
			this.setNotificationData(new AlertDataModel(rawData.alertData));
		}
	}

	getData() {
		return {
			_id       : this._key,
			masterKey : this._masterKey,
			clientKey : this._clientKey,
			lastData         : this._lastData ? this._lastData.getData() : null,
			notificationData : this._notificationData ? this._notificationData.getData() : null,
			alertData        : this._alertData ? this._alertData.getData() : null,
			safePlaces       : this._safePlaces || []
		}
	}

	getKey() {
		return this._key;
	}

	getLastData() {
		return this._lastData;
	}

	setLastData(record) {
		if (record.validate() === true) {
			this._lastData = record;
		}
	}

	getSafePlaces() {
		return this._safePlaces || [];
	}

	setSafePlaces(safePlaces) {
		this._safePlaces = safePlaces;
	}

	getNotificationData() {
		return this._notificationData;
	}

	setNotificationData(record) {
		if (record.validate() === true) {
			this._notificationData = record;
		}
	}

	getAlertData() {
		return this._alertData;
	}

	setAlertData(record) {
		if (record.validate() === true) {
			this._alertData = record;
		}
	}

	_defaultData() {
		this._masterKey = null;
		this._clientKey = null;
		this._key = null;
	}

	_validateKey(reference, key) {
		if (!reference) return false;
		return SecUtil.comparePassword(key, reference);
	}

	updateClientKey(rawKey) {
		this._clientKey = SecUtil.generatePasswordString(SecUtil.salt().replace("=", ""), "sha256", rawKey);
	}

	hasClientAccess(key) {
		return this._validateKey(this._clientKey, key) || this.hasMasterAccess(key);
	}

	hasMasterAccess(key) {
		return this._validateKey(this._masterKey, key);
	}
}

module.exports = BagModel;