'use strict';

class BagUpdateDataModel {
	constructor(rawData) {
		this._defaultData();
		if (rawData) {
			this.parseData(rawData);
		}
	}

	parseData(rawData) {
		this._defaultData();
		this._setRecordIfPresent("ts", rawData);
		this._setRecordIfPresent("lon", rawData);
		this._setRecordIfPresent("lat", rawData);
		this._setRecordIfPresent("battery", rawData);
		this._setRecordIfPresent("phone_active", rawData);
		this._setRecordIfPresent("missing_gadgeds", rawData);
		this._setRecordIfPresent("gadgets", rawData);
	}

	getData() {
		return {...this._record};
	}

	_defaultData() {
		this._record = {
			ts : 0
		}
	}

	getTimeStamp() {
		return this._record["ts"] || 0;
	}

	validate() {
		if (Object.keys(this._record).length === 0) {
			return "Data is empty";
		}

		if (!this._record["ts"] || this._record["ts"] === 0) {
			return "Timestamp is mandatory, but not present";
		}

		return true;
	}

	_setRecordIfPresent(key, rawData) {
		if (rawData[key] !== undefined) {
			this._record[key] = rawData[key];
		}
	}
}

module.exports = BagUpdateDataModel;
