'use strict';
const CONST = require("../constants.js");
const LOG   = require("./Logger.js");

const Configuration = require("./Configuration.js");

class ServiceBase {
	constructor(config) {
		this._config        = config || new Configuration();
		this._lastInterrupt = 0;
		this._shutdown      = false;

		//setting up OS signal handlers
		process.on("SIGTERM", () => this.sigTERM());
		process.on("SIGINT", () => this.sigINT());
	}

	start() {
		if (this._failOnMissingconfig() && !this._config.hasConfig()) {
			LOG.error("Cannot start service, no valid configuration was loaded");
			return
		}
		LOG.info("Initialize");
		this._initialize(() => {
			LOG.log("Starting service");
			this._run();
		});
	}

	_initialize(done) {
		done();
	}

	_run() {

	}

	_terminate(done) {
		done();
	}

	_failOnMissingconfig() {
		return false;
	}

	stop() {
		LOG.log("Stopping service...");
		this._shutdown = true;
		this._terminate(() => {
			LOG.log("Exiting");
			//process.exit(0);
		});
	}

	sigTERM() {
		LOG.log("Received SIGTERM...");
		this.stop();
	}

	sigINT() {
		let now = Date.now();
		if (now - this._lastInterrupt <= CONST.INTERRUPT_WINDOW_SECONDS * 1000) {
			if (!this._shutdown) {
				LOG.log("Received second SIGINT within the interrupt window, stopping.");
				this.stop();
			} else {
				LOG.error("Received third SIGINT within the interrupt window, forcefully stopping, sorry for the inconvenience.");
				process.exit(1);
			}
		} else {
			LOG.log("Received SIGINT... press CTRL + C again in " + CONST.INTERRUPT_WINDOW_SECONDS + " seconds to terminate");
		}

		this._lastInterrupt = now;
	}
}

module.exports = ServiceBase;