const EventEmitter = require('events')
const PN532 = require('./pn532-spi.lib.js')
const pn532 = new PN532({
	clock:  23, 	// SCLK
	mosi:   19, 	// MOSI
	miso:   21, 	// MISO
	client: 24, 	// SSEL
	debug: false	// Enable to get debug logging for frames
})

let bytesToHex = function(arr) {
	return arr.reduce(function(a, b) {
		let result = b.toString(16)
		if (result.length == 1) result = '0' + result
		return a + result
	}, '0x')
}

module.exports = class PN532 extends EventEmitter {

	// Init the shield for scanning
	constructor () {

		super()

		this.version = ''
		this.active = false
		this.scan_timer = null
		this.starter_timer = null

		this.init()
	}

	// Will throw an error if PN532 is unresponsive or misconfigured
	init () {
		pn532.begin()

		this.version = pn532.getFirmwareVersion()
		console.log('PN532 Firmware version: ', this.version[1] + '.' + this.version[2])

		// Configure PN532 for Mifare cards
		pn532.samConfiguration()
	}

	// Perform an NFC / RFID scan
	scan () {
		let uid = pn532.readPassiveTarget()
		if (uid === null) return

		// Dispatch scan event
		this.emit('scan', bytesToHex(uid));

		// Wait for 2 seconds before attempting to scan again
		clearInterval(this.scan_timer)
		this.starter_timer = setTimeout(this.scan_start.bind(this), 2000)
	}

	// Start scanning (returns after the last scan has concluded)
	scan_start () {
		this.active = true
		this.scan_timer = setInterval(this.scan.bind(this), 250)
	}

	// Stop scanning (returns after the last scan has concluded)
	scan_stop () {
		clearTimeout(this.starter_timer)
		clearInterval(this.scan_timer)
	}

}
