const SerialPort = require('serialport'),
	  ReadLine = require('@serialport/parser-readline'),
	  Hash = require('object-hash')

var use_ports = {
	sim808: { path: '/dev/ttyS0' }
}

module.exports = class Serial {

	// Konstruktor
	constructor () {

		this.ports = []

		this.check()
		setInterval(this.check.bind(this), 10000)

		// Stream interfészek készítése
		for (let key in use_ports) {

			let port_cfg = use_ports[key]

			let port = new SerialPort(port_cfg.path, {
				autoOpen: false,
				echo: true,
				baudRate: 115200
			})

			port.on('error', (err) => {
				console.log(key + ' produced an error: ', err)
				this.check()
			})

			port.on('close', () => {
				console.log(key + ' is now closed')
				this.check()
			})

			port.parser = port.pipe(new ReadLine({ delimiter: String.fromCharCode(13) }))

			this[key] = port
		}
	}

    check () {

		// Check for available ports
        SerialPort.list((err, serialports) => {

            if (Hash(serialports) != Hash(this.ports)) {
				// There are changes compared to the last port list state
                this.ports = serialports

				// Update all ports in config
				for (let key in use_ports) {

					let port_cfg = use_ports[key]

					// Check to see if the port is available..
					if (this.ports.find((entry) => { return entry.comName == port_cfg.path })) {
						// ..it is

						// If the port is closed, open it
						if (!this[key].isOpen) this[key].open()
					} else {
						// ..it is not

						// If the port is open, close it
						if (this[key].isOpen) this[key].close()
					}
				}
            }
        })
    }
}
