'use strict';
const CONST  = require("../../constants.js");
const LOG    = require("../../core/Logger.js");

const UserBasedHandlerBase = require("./UserBasedHandlerBase.js");
const HandlerException = require("./HandlerException.js");

class ActivationHandler extends UserBasedHandlerBase {
	static getRoute() {
		return "/user/:userId/activate/:token";
	}

	async _processGET() {
		return this._dal.updateUserData(this._user.getId(), this._user.getData())
			.then(r => this.redirect("/"));
	}

	async _setup() {
		return this._dal.getUserData(this._vars.userId)
			.then(res => {
				if (!res) return Promise.reject(this.respondNotFound("Unknown user"));
				this._user = res;
			});
	}

	_refreshAuthCookie() {
		this._setCookie("smbuiauth", this._crypto.encrypt(JSON.stringify(this._auth)), CONST.AUTH_TTL_SEC);
	}

	//The only reason to override it is to make sure nothing is relying on auth in the endpoint
	_getAuth() {
		return null;
	}

	//This is a non-auth endpoint.
	async _isAuthenticated() {
		return true;
	}

	async _hasAccess() {
		try {
			return this._user.activate(this._vars.token);
		} catch (e) {
			LOG.error("Error during user activation", e);
			return false;
		}
	}
}

module.exports = ActivationHandler;