'use strict';
const CONST = require("../constants.js");

class Logger {
	constructor(logLevel, outputHandler) {
		this.setLogLevel(logLevel);

		this._out = {
			info  : () => {},
			log   : () => {},
			warn  : () => {},
			error : () => {}
		}

		this.err = this.error;
		this.warn = this.warning;
		this.log  = this.info;
	}

	setLogLevel(logLevel) {
		this._logLevel = logLevel || CONST.DEFAULT_LOG_LEVEL;
		return this;
	}

	setOutputChannel(out) {
		this._out = out;
		return this;
	}

	_padz(str, num) {
		return ("" + str).padStart(num, "0");
	}

	_prepArgs(args) {
		let now = new Date();
		let prefix =
			"[" +
			now.getFullYear() + "-" +
			this._padz(now.getMonth() + 1, 2) + "-" +
			this._padz(now.getDate(), 2) + " " +
			this._padz(now.getHours(), 2) + ":" +
			this._padz(now.getMinutes(), 2) + ":" +
			this._padz(now.getSeconds(), 2) + "." +
			this._padz(now.getMilliseconds(), 3) +
			"]";

		return [prefix, ...args];
	}

	debug(...args) {
		if (this._logLevel > this.LEVEL_DEBUG) return;
		this._out.info.apply(this._out, this._prepArgs(args));
	}

	info(...args) {
		if (this._logLevel > this.LEVEL_INFO) return;
		this._out.log.apply(this._out, this._prepArgs(args));
	}

	warning(...args) {
		if (this._logLevel > this.LEVEL_WARNING) return;
		this._out.warn.apply(this._out, this._prepArgs(args));
	}


	error(...args) {
		this._out.error.apply(this._out, this._prepArgs(args));
	}
}

Logger.prototype.LEVEL_DEBUG = 1;
Logger.prototype.LEVEL_ALL   = 1;
Logger.prototype.LEVEL_INFO  = 2;
Logger.prototype.LEVEL_WARNING = 3;
Logger.prototype.LEVEL_ERROR = 4;

const logger = new Logger();

module.exports = logger;