const React     = require("react");
const {connect} = require("react-redux");

const { createMuiTheme, makeStyles, ThemeProvider } = require('@material-ui/core/styles');
const Snack = require("./Snack.jsx").default;
const SmbfwActions = require("../Action/SmbfwActions.js");
const ScreenLock = require("./ScreenLock.jsx").default;

const Application = (props) => {
	const smbfw    = props.smbfw;
	const screen   = props.currentScreen;
	const modal    = props.modalWindow;
	const snack    = props.snack;
	const locks    = props.locks;

	//error handling, for invalid screen we don't show anything, perhaps a loading screen?
	if (!screen) return React.createElement("div", null, null);

	let elements = [
		React.createElement(screen.getScreen(), null, null),
	]

	if (modal) elements.push(
		React.createElement(modal.getModal(), null, null)
	);

	if (snack.open) elements.push(
		React.createElement(Snack, {
			onClose : props.closeSnack,
			...snack
		}, null)
	);

	if (locks.full > 0) elements.push(
		React.createElement(ScreenLock, {
			open : (locks.full > 0)
		})
	);



	let theme = createMuiTheme({
		palette: {
			type: 'dark',
			primary: {
				main: "#F1A21F",
			},
			secondary: {
				main: "#EEE",
			},
		},
	});

	return React.createElement(ThemeProvider, {theme : theme},
		React.createElement("div", { "className":"application " + (props.mainCSS || "")},
			...elements
		)
	);
}

const mapStateToProps = (state) => ({
    currentScreen : state.smbfw.currentScreen,
    modalWindow   : state.smbfw.modalWindow,
    smbfw         : state.smbfw.smbfw,
    mainCSS       : state.smbfw.mainCSS,
    snack         : state.smbfw.snack || {open : false},
    locks         : state.smbfw.lock || {}
});

const mapDispatchToProps = {
	"closeSnack" : SmbfwActions.closeSnack
};

module.exports = connect(mapStateToProps, mapDispatchToProps)(Application);
