'use strict';

const _createDataPipeError = (code) => {
	let fn = (msg, params) => {
		params = (params === null || typeof params !== "object") ? {} : params;
		return {
			code   : code,
			msg    : msg,
			params : params
		}
	}
	fn.check = (other) => other && (other.code === code);

	return fn;
};

const DataPipeError = {
	//Should be returned when on load the desired index is not found
	notFound         : _createDataPipeError("notFound"),
	//Should be used when trying to create an object with an already existing ID
	alreadyExists    : _createDataPipeError("alreadyExists"),
	//Should be used when the data to be updated have been modified since the query
	hasBeenModified  : _createDataPipeError("hasBeenModified"),
	//Should be used on generic data access errors, like connection or permission issues
	dataAccessError  : _createDataPipeError("dataAccessError"),
	//this error should be used for all non specific errors
	otherError       : _createDataPipeError("otherError")
};

module.exports = DataPipeError;