'use strict';

class RESTPathSplitter {
	constructor(createPath, createMethod, savePath, saveMethod, loadPath, loadMethod) {
		this._createPath   = createPath;
		this._createMethod = createMethod;
		this._savePath     = savePath;
		this._saveMethod   = saveMethod;
		this._loadPath     = loadPath;
		this._loadMethod   = loadMethod;
	}

	_prepareOptions(options, method, path) {
		if (!options) options = {};
		options.method      = method;
		options.pathPattern = path;
		options.savePath    = this._savePath;
		options.saveMethod  = this._saveMethod;
		return options;
	}

	preSaveData(id, data, options) {
		if (!this._saveMethod) throw new Error("Save operation is not supported in this endpoint!");
		options = this._prepareOptions(options, this._saveMethod, this._savePath);
		return [id, data, options];
	}

	preLoadData(id, options) {
		if (!this._loadMethod) throw new Error("Load operation is not supported in this endpoint!");
		options = this._prepareOptions(options, this._loadMethod, this._loadPath);
		return [id, options];
	}

	preCreateData(data, options) {
		if (!this._createMethod) throw new Error("Create operation is not supported in this endpoint!");
		options = this._prepareOptions(options, this._createMethod, this._createPath);
		return [data, options];
	}

	postSaveData(id, data, options) {
		return [id, data.data, options];
	}

	postLoadData(id, data, options) {
		return [id, data.data, options];
	}

	postCreateData(id, data, options) {
		return [id, data.data, options];
	}
}

module.exports = RESTPathSplitter;
