'use strict';
const ManagerBase       = require("./ManagerBase.js");

const {
	InvalidStateException,
	InvalidArgumentException,
	InvalidStateChangeException
} = require('../Exception/Exceptions.js');

class ActionHandlerManager extends ManagerBase {
	constructor(name, mandatoryPrefix, supressWithPrefix) {
		super();

		this._name = name;
		this._info("Action handler manager created:", name);

		this._prefix       = mandatoryPrefix || false;
		this._prefixLength = this._prefix ? this._prefix.length : 0;
		this._supress      = supressWithPrefix || false;

		this._handlers = [];
	}

	handle(action, next, state) {
		var supress = false;

		if (this._prefix) {
			let actionPrefix = action.type.substr(0, this._prefixLength);
			if (actionPrefix === this._prefix) {
				if (!action.nolog) this._debug("Action handler prefix match",this._name, action);
				supress = this._supress;
			} else {
				if (!action.nolog) this._debug("Action prefix does not match this handler, skipping", this._prefix, "vs", actionPrefix);
				next(action);
				return;
			}
		} else {
			if (!action.nolog) this._debug("Action handler",this._name, action);
		}

		var fakeNext = (a) => {action = a};
		var oldAction;
		var nolog = action.nolog;
		for (let i = 0; i < this._handlers.length; i++) {
			let currentHandler = this._handlers[i];
			try {
				oldAction = action;
				action    = null;
				currentHandler(oldAction, fakeNext, state);
			} catch (e) {
				this._error(e);
			}

			//if the current handler haven't returned any action, we just return, since there is nothing to handle anymore
			if (!action) {
				if (!nolog) this._debug("Action has been supressed by handler #", i);
				return;
			}
		}

		//at this point we must have an action, calling next middleware
		if (!supress) {
			next(action);
		} else {
			this._debug("Supressing handler exclusive action");
		}
	}

	generateHandler() {
		return state => next => action => {
			return this.handle(action, next, state);
		}
	}

	addHandler(handler) {
		//TODO Check proper parent class
		this._handlers.push(handler);
	}
}

ActionHandlerManager.prototype.PUBLISHED_METHODS = [];
module.exports = ActionHandlerManager;