'use strict';
const SmbfwObjectBase = require("../SmbfwObjectBase.js");
const ManagedObjectBase  = require("../Components/ManagedObjectBase.js");

const {
	UnimplementedMethodException,
	InvalidArgumentException,
	MissingFieldException
} = require('../Exception/Exceptions.js');

class ManagerBase extends SmbfwObjectBase {
	constructor() {
		super();

		//the managed object collection
		this._managed = {};

		this._dispatch = null;
	}

	_addManaged(managableClass) {
		if (managableClass === null || managableClass === undefined) throw new InvalidArgumentException("The registerScreen method expects a CLASS derived from " + this.util.getClassname(this.MANAGED_BASE_CLASS));
		if (!(managableClass.prototype instanceof this.MANAGED_BASE_CLASS)) throw new InvalidArgumentException("The registerScreen method expects a CLASS derived from " + this.util.getClassname(this.MANAGED_BASE_CLASS) + ". Instead it received: " + this.util.getClassname(managableClass));

		var object = this._smbfw.createSmbfwObject.apply(this._smbfw, arguments);
		var name = object.getManagedName();

		if (this._managed[name]) {
			this._warn("Object with key '" + name + "' is already registered.");
			return this._managed[name];
		}

		this._info("Registering '" + name + "'");
		this._managed[name] = object;
		object._setManager(this);

		return object;
	}

	_getManaged(name) {
		return this._managed[name] ? this._managed[name] : null;
	}

	generateAccessors() {
		var methods = Object.getPrototypeOf(this).PUBLISHED_METHODS;
		var ret     = {};
		if (!methods) throw new MissingFieldException(this.util.getClassname(this) + " must define the PUBLISHED_METHODS static field or override the generateAccessors method!");
		if (!Array.isArray(methods)) throw new MissingFieldException(this.util.getClassname(this) + " must define the PUBLISHED_METHODS static field as an ARRAY of the exposed method names!");

		for (let i in methods) {
			let method  = methods[i];
			ret[method] = this[method].bind(this);
		}

		return ret;
	}

	setDispatch(dispatch) {
		this._dispatch = dispatch;
	}

	preStart() {
		//Placeholder to make sure, all managers have preStart methods
	}

	start() {
		//Placeholder to make sure, all managers have start methods
	}
}

ManagerBase.prototype.MANAGED_BASE_CLASS = ManagedObjectBase;
ManagerBase.prototype.PUBLISHED_METHODS  = null;

module.exports = ManagerBase;