'use strict';
const ManagerBase       = require("./ManagerBase.js");
const SmbfwActions   = require("../Action/SmbfwActions.js");

class SmbfwServicesManager extends ManagerBase {
	_init() {
		this._serviceHandlerMap = {};

		this._serviceHandlerMap[SmbfwActions.changeMainCSS.type] = this.handleChangeMainCSS.bind(this);
		this._serviceHandlerMap[SmbfwActions.fetchDataToStore.type] = this.handleFetchDataToStore.bind(this);
	}

	businessLogicActionHandler(action, next, state) {
		let parts = action.type.split(".");

		if (parts[2]) {
			let bl = this._getManaged(parts[2]);

			if (bl) bl.handle(action, state, this._dispatch);
		}
	}

	serviceActionHandler(action, next, state) {
		let handler = this._serviceHandlerMap[action.type];

		if (handler) {
			handler(action, next, state);
		}
	}

	handleChangeMainCSS(action, next, state) {
		this._dispatch(SmbfwActions.updateState({
			"smbfw.mainCSS" : action.payload
		}));
	}

	handleFetchDataToStore(action, next, state) {
		this._smbfw.dm.get(action.payload.dataSource, action.payload.key).then(data => {
			this._dispatch(SmbfwActions.updateState({
				[action.payload.storePath] : data.getData()
			}));
		}).catch(e => this._error(e));
	}
}

SmbfwServicesManager.prototype.PUBLISHED_METHODS  = [];
//SmbfwServicesManager.prototype.MANAGED_BASE_CLASS = BusinessLogicBase;
module.exports = SmbfwServicesManager;