'use strict';

class Logger {
	constructor() {
		this._logLevel = 1;
		this.debug = args => {
			if (this._logLevel > 0) return;
			console.debug.apply(console, args);
		}

		this.info  = args => {
			if (this._logLevel > 1) return;
			console.info.apply(console, args);
		};

		this.log   = args => {
			if (this._logLevel > 2) return;
			console.log.apply(console, args);
		}
		this.warn  = args => {
			if (this._logLevel > 3) return;
			console.warn.apply(console, args);
		};
		this.error = args => {
			if (this._logLevel > 4) return;
			console.error.apply(console, args);
		};
	}

	setLogLevel(level) {
		this._logLevel = level;
	}

	createChannel(name) {
		var self = this;
		return new LogChannel(name, this);
	}

}

class LogChannel {
	constructor(name, logger) {
		this._name   = name;
		this._logger = logger;
		this._formatter = this._formatModule;
	}

	_timestamp() {
		return (new Date()).toISOString();
	}

	_formatLog(args) {
		return ["[" + this._timestamp() + "|" + this._name + "]", ...args];
	}

	_simpleFormat(args) {
		return args;
	}

	_formatModule(args) {
		return ["[" + this._name + "]", ...args];
	}

	debug(...args) {
		this._logger.debug(this._formatter(args))
	}

	log(...args) {
		this._logger.log(this._formatter(args))
	}

	info(...args) {
		this._logger.info(this._formatter(args))
	}

	warn(...args) {
		this._logger.warn(this._formatter(args))
	}

	error(...args) {
		this._logger.error(this._formatter(args))
	}

	bindToObject(object) {
		object._debug = this.debug.bind(this);
		object._info  = this.info.bind(this);
		object._log   = this.log.bind(this);
		object._warn  = this.warn.bind(this);
		object._error = this.error.bind(this);
	}

}

const _logger = new Logger();

module.exports = _logger;