"use strict";
const BusinessLogicBase = require("../../smbfw/src/smbfw.js").BusinessLogicBase;
const SmbfwActions      = require("../../smbfw/src/smbfw.js").Actions;

class DataManagerBL extends BusinessLogicBase {
	constructor() {
		super();
		this._fetcher = null;
	}

	handleStartFetch(state, dispatch)	{
		if (this._fetcher === null) {
			this._fetcher = setInterval(() => this._periodicFetcher(dispatch), 10000);
			this._periodicFetcher(dispatch);
		}
	}

	handleStopFetch(state, dispatch)	{
		if (this._fetcher !== null) {
			clearInterval(this._fetcher);
			this._fetcher = null;
		}
	}

	_periodicFetcher(dispatch) {
		dispatch(DataManagerBL.Actions.actionRefreshData());
	}

	handleRefreshData(state, dispatch) {
		let user = this.util.resolveReference("dynamic.user", state, null);
		if (user) {
			this._smbfw.dm.loadData("data_fetch", null, user.getBagId())
				.then(bag => dispatch(
					SmbfwActions.updateState({
						"dynamic.bag": bag
					})
				));
		}
	}

}

DataManagerBL.prototype.MANAGED_NAME = "DataManagerBL";

BusinessLogicBase.generateActions(DataManagerBL, {
	actionStartFetch : "handleStartFetch",
	actionStopFetch : "handleStopFetch",
	actionRefreshData : "handleRefreshData"
});

module.exports = DataManagerBL;
