const React     = require("react");
const connect   = require("../../../smbfw/src/smbfw.js").connect;
const Util      = require("../../../smbfw/src/smbfw.js").Util;
const LayoutScreenBase     = require("./LayoutScreenBase.jsx").default;
const NotificationConfigScreenActions = require("../../businessLogic/NotificationConfigScreenBL.js").Actions;

import FormControlLabel from '@material-ui/core/FormControlLabel';
import Switch from '@material-ui/core/Switch';
import Button from '@material-ui/core/Button';
import Paper from '@material-ui/core/Paper';
import Typography from '@material-ui/core/Typography';


class NotificationConfigScreen extends LayoutScreenBase {
	constructor(props) {
		super(props);
		this.state = {
			no_phone       : props.config ? props.config.get("no_phone") : false,
			missing_gadget : props.config ? props.config.get("missing_gadget") : false,
			low_battery    : props.config ? props.config.get("low_battery") : false
		};
	}

	_genericStateUpdate(event) {
		if (event.target.type === "checkbox") {
			this.setState({ [event.target.name]: event.target.checked });
		} else {
			this.setState({ [event.target.name]: event.target.value });
		}
	}

	_addToggle(variable) {
		return (
			<FormControlLabel
				control={
					<Switch
						checked={this.state[variable]}
						onChange={e => this._genericStateUpdate(e)}
						name={variable}
						color="primary"
					/>
				}
				style={{
					width : "100%"
				}}
				label={this.props.$_l("screen.notification_config." + variable)}
			/>
		)
	}

	renderScreen() {
		let $_l = this.props.$_l;
		return (
			<Paper className="form" style={{
				margin: "20px auto 20px auto",
				width: "40%",
				padding: "20px",
			}}>
				<Typography variant="h3" component="h2" style={{marginBottom : "20px"}}>
					{$_l("screen.notification_config.title")}
				</Typography>
				<Typography variant="body1" style={{marginBottom : "20px"}}>
					{$_l("screen.notification_config.info")}
				</Typography>
				{this._addToggle("no_phone", $_l("screen.notification_config.no_phone"))}
				<br/>
				{this._addToggle("missing_gadget", $_l("screen.notification_config.missing_gadget"))}
				<br/>
				{this._addToggle("low_battery", $_l("screen.notification_config.low_battery"))}
				<div style={{justifyContent: "flex-end", display: "flex"}}>
					<Button variant="contained"
						color="primary"
						style={{
							marginTop : "40px"
						}}
						onClick={() => this.props.actionUpdateNotificationSettings(
							this.state.no_phone,
							this.state.missing_gadget,
							this.state.low_battery
						)}
					>
						{$_l("shared.save")}
					</Button>
				</div>
			</Paper>
		);
	}
}

const mapStateToProps = state => ({
	config : Util.resolveReference("dynamic.notificationConfig", state, null),
	...(LayoutScreenBase.getStateToProps(state))
});

const mapDispatchToProps = NotificationConfigScreenActions.toMapDispatchToProps({
	...(LayoutScreenBase.getDispatchToProps())
});

export default connect(
	mapStateToProps,
	mapDispatchToProps
)(NotificationConfigScreen);
