"use strict";
const Smbfw = require("../smbfw/src/smbfw.js");
const RESTProvider     = Smbfw.DataProvider.RESTProvider;
const LoginScreenBL    = require("./businessLogic/LoginScreenBL.js");

var smbfw = new Smbfw();
window.startApplication = function() {
	smbfw.l.setLocalizationData(require("./localization.js"));
	smbfw.l.setLocale("hu");
	smbfw.ss.setInitialState({
		static : {},
		dynamic : {}
	});

	// ----- Screen registration -----
	smbfw.sm.registerScreen(
		require("./components/screen/LoginScreenController.js"),
		require("./components/screen/RegistrationScreenController.js"),
		require("./components/screen/NotificationConfigScreenController.js"),
		require("./components/screen/ProfileScreenController.js"),
		require("./components/screen/ForgottenPasswordScreenController.js"),
		require("./components/screen/TrackBagScreenController.js"),
		require("./components/screen/DashboardScreenController.js")
	);

/*
	smbfw.sm.registerModal(
		require("./components/modal/GenericModalWindow/GenericModalWindowController.js")
	);
*/
	smbfw.sm.setInitialScreen("login_screen");
	smbfw.setInitialAction(LoginScreenBL.Actions.autoLogin);

	// ----- Business logic registration -----
	smbfw.bl.registerBusinessLogic(
		require("./businessLogic/LoginScreenBL.js"),
		require("./businessLogic/RegistrationScreenBL.js"),
		require("./businessLogic/ProfileScreenBL.js"),
		require("./businessLogic/NotificationConfigScreenBL.js"),
		require("./businessLogic/NavBarBL.js"),
		require("./businessLogic/TrackBagScreenBL.js"),
		require("./businessLogic/ForgottenPasswordScreenBL.js"),
		require("./businessLogic/DashboardScreenBL.js"),
		require("./businessLogic/DataManagerBL.js")
	);

	smbfw.ss.setInitialState({
	});

	const RESTModelBase = Smbfw.RESTModelBase;
	const fetchBagToken = smbfw.ss.createFetchMethod("dynamic.user", {getClientToken : () => ""})

	let webApiBuilder = smbfw.dm.createRESTBuilder(location.protocol + "//" + location.host + "/web", [
			Smbfw.DataPipeMiddleware.ObjectToJson,
			new Smbfw.DataPipeMiddleware.DynamicRESTHeaderAppender(() =>({
				"Content-Type": "application/json"
			}))
		],
		new Smbfw.DataProvider.RESTProvider(require('request-promise-native'), 10000)
	);

	let dataApiBuilder = smbfw.dm.createRESTBuilder(location.protocol + "//" + location.host + "/data", [
			Smbfw.DataPipeMiddleware.ObjectToJson,
			new Smbfw.DataPipeMiddleware.DynamicRESTHeaderAppender(() =>({
				"Content-Type": "application/json",
				"x-smb-auth-token": fetchBagToken().getClientToken()
			}))
		],
		new Smbfw.DataProvider.RESTProvider(require('request-promise-native'), 10000)
	);

	webApiBuilder
		.connectModel("register", require("./models/UserRDM.js"), "/user/register")
			.create("", "POST")
		.connectModel("user", require("./models/UserRDM.js"), "/user")
			.load("/:id", "GET")
			.save("/:id", "POST")
		.connectModel("authenticate", require("./models/AuthenticateRDM.js"), "/user")
			.create("/:id/authenticate", "PUT")
		.connectModel("reset", require("./models/ResetPasswordRDM.js"), "/user")
			.create("/:id/reset", "POST")
		.registerPipes();

	dataApiBuilder
		.connectModel("notification_config", require("./models/NotificationConfigRDM.js"), "/:bagId/alert")
			.load("", "GET")
			.save("", "PUT")
		.connectModel("data_fetch", require("./models/BagDataRDM.js"), "/:bagId/data")
			.load("", "GET")
		.registerPipes();
	try {
		smbfw.start("root");
	} catch (e) {
		console.error(e);
	}
}